/*
** Contains prototype of engine functions.
**
** $Id: ant_engine.h,v 1.20 2001/02/24 22:54:41 kilobug Exp $
*/

#ifndef __ANT_ENGINE_H__
#define __ANT_ENGINE_H__

#ifndef __ANT_H__
#error "MUST BE INCLUDED FROM ANT.H"
#endif

extern GMutex *ants_mutex;

#define ant_enter_critical() g_mutex_lock(ants_mutex)
#define ant_leave_critical() g_mutex_unlock(ants_mutex)

/******************
 ** Initializers **
 ******************/

/* Main function for static linked clients */
error_t ant_main(int argc, char **argv, ant_callback_t *live, 
		 ant_callback_t *born, ant_callback_t *exit);

/* Main function for the "ants" .so dynamic-loader client */
error_t ant_main_module(int argc, char **argv);

/* Ant engine initializer: parse command line, load rules, create the world */
world_t *ant_engine_init(int argc, char **argv,ant_callback_t *live, 
			 ant_callback_t *born, ant_callback_t *exit);

/*******************
 ** Ants handling **
 *******************/

/* Create a new ant */
error_t ant_create_ant(world_t *w);

/* Callback for the pthread_create function */
void *ant_new_ant(ant_t *ant);

/* Kill the current ant: destroy the thread, free memory, ... */
void ant_kill_ant(ant_t *ant);

/* Send the KILL signal to all ants */
void ant_kill_all_ants(world_t *w);

/*************
 ** General **
 *************/

/* Wait */
void ant_apply_delay(const delay_t *delay, world_t *w);

/* Parse a config file */
void ant_parse_conf_file(rules_t *rules, const char *self);

/* Parse a map file */
void ant_load_map(const char *name, world_t *world);

/* Get an unique number - thread safe */
int ant_get_uniq_id();

#endif
