/*
** The GTK+ interface: control functions.
**
** $Id: ant_gtk.c,v 1.10 2001/02/24 23:41:47 kilobug Exp $
*/

#include "ant_gtk.h"

int ant_gtk_ready = FALSE;

/* Bye bye */
void ant_destroy(GtkWidget *widget, gpointer data)
{
  ant_gtk_thread_lock();
  gtk_main_quit();
}

/* Display a message in the status bar */
void ant_disp_msg(const char *fmt, ...)
{
  va_list ap;
  static char buf[420];
  static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
  int i;

  g_static_mutex_lock(&mutex);
  va_start(ap, fmt);
  i = g_vsnprintf(buf, 420, fmt, ap);
#ifdef __DEBUG__
  printf("%s", buf);
#endif
  if (buf[i - 1] == '\n')
    buf[i - 1] = 0;
  gdk_threads_enter();
  ant_set_status(buf);
  gdk_threads_leave();
  va_end(ap);
  g_static_mutex_unlock(&mutex);
}

/* The main loop: creates widgets and calls gtk_main() */
void ant_main_loop_gtk(world_t *w)
{
  GtkWidget *window = NULL;
  GtkWidget *box = NULL;
  
  /* Pause the whole system by default */
  ant_enter_critical();
  ant_gtk_thread_lock();

  /* Create window */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(window), "destroy", 
		     GTK_SIGNAL_FUNC(ant_destroy), NULL);
  gtk_window_set_title(GTK_WINDOW(window), "Ant simulator");

  /* Add objects */
  box = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(window), box);  
  gtk_box_pack_start(GTK_BOX(box), ant_make_menubar(window, w), 
		     TRUE, TRUE, 0);  
  gtk_box_pack_start(GTK_BOX(box), ant_make_mainbox(window, w), 
		     TRUE, TRUE, 0);  
  gtk_box_pack_start(GTK_BOX(box), ant_make_statusbar(window), TRUE, TRUE, 0);
  gtk_widget_show(box);
  gtk_widget_show(window);

  /* Main loop */
  gdk_threads_enter();
  ant_leave_critical();
  ant_gtk_ready = TRUE;
  gtk_main();
  gdk_threads_leave();
  gtk_exit(0);
}

/* Creates the mutex and calls gtk_init */
void ant_gtk_loop_init(int *argc, char ***argv)
{
  gtk_init(argc, argv);
  ant_gtk_thread_init_mutex();
}
