/*
** Private .h for GTK thread.
**
** $Id: ant_gtk.h,v 1.11 2001/02/24 23:41:47 kilobug Exp $
*/

#ifndef __ANT_GTK_H__
#define __ANT_GTK_H__

#include "ant.h"
#include "ant_private.h"
#include <stdarg.h>
#include <gdk/gdkkeysyms.h>

extern int ant_gtk_ready;

/*********************
 ** Mutex functions **
 *********************/

/* Create the mutex */
void ant_gtk_thread_init_mutex();

/* Enter pause mode */
void ant_gtk_thread_lock();

/* Leave pause mode */
void ant_gtk_thread_unlock();

/* Callback wrapper to ant_gtk_thread_unlock() */ 
void ant_gtk_unlock_callback(GtkWidget *w, gpointer data);

/* Callback wrapper to ant_gtk_thread_lock() */ 
void ant_gtk_lock_callback(GtkWidget *w, gpointer data);

/* Allow just one step */
void ant_gtk_step_callback(GtkWidget *w, gpointer data);


/***********************
 ** General callbacks **
 ***********************/

/* Destroy target widget */
void ant_destroy_widget(GtkWidget *w, gpointer data);

/* Hide target widget */
void ant_hide_widget(GtkWidget *w, gpointer data);

/* Bye bye */
void ant_do_exit(GtkWidget *w, gpointer data);

/* Config panel callbacks */
void ant_set_data_from_entry(GtkWidget *w, char **data);
void ant_set_data_from_toggle(GtkWidget *w, int *data);
void ant_set_data_from_spin(GtkWidget *w, int *data);

/***************
 ** Interface **
 ***************/

/* Create the menu bar */
GtkWidget *ant_make_menubar(GtkWidget *window, world_t *w);

/* Create the main box with stats and map */
GtkWidget *ant_make_mainbox(GtkWidget *window, world_t *w);

/* Create the status bar */
GtkWidget *ant_make_statusbar(GtkWidget *window);

/* Set the message in the status bar */
void ant_set_status(const char *msg);

/* Display the about dialog box */
void ant_gtk_disp_about(GtkWidget *w, gpointer data);

/* Display the options dialog box */
void ant_gtk_show_options(GtkWidget *wid, gpointer data);

/* Display the stats dialog box */
void ant_gtk_disp_stat(GtkWidget *wi, world_t *w);

/* Redraw the full world */
void ant_gtk_redraw_world(world_t *w);

#endif
