/*
** The GTK+ interface: interface creating functions.
**
** $Id: ant_gtk_mainbox.c,v 1.14 2001/02/24 23:41:47 kilobug Exp $
*/

#include "ant_gtk.h"
#include "ant_gtk_mainbox.h"


GdkGC *ant_gfx_get_color(GdkColormap *gdkcolormap, 
			 int red, int green, int blue,
			 GdkDrawable *drawable)
{
  GdkGC *res;
  GdkColor gdkcolor;

  gdkcolor.pixel = 0;
  gdkcolor.red = red;
  gdkcolor.green = green;
  gdkcolor.blue = blue;
  gdk_colormap_alloc_color(gdkcolormap, &gdkcolor, TRUE, TRUE);
  res = gdk_gc_new(drawable);
  gdk_gc_set_foreground(res, &gdkcolor);
  return res;
}

void ant_gfx_get_colors(ant_gfx_drawing_t *d)
{
  GdkColormap *gdkcolormap;

  gdkcolormap = gdk_colormap_get_system();  
  d->line = d->drawing_area->style->black_gc;
  d->back = d->drawing_area->style->white_gc;
  d->entry = ant_gfx_get_color(gdkcolormap, 0, 0, 0xffff, 
			       d->drawing_area->window);
  d->exit = ant_gfx_get_color(gdkcolormap, 0xffff, 0, 0, 
			      d->drawing_area->window);
  d->home = ant_gfx_get_color(gdkcolormap, 0, 0xffff, 0,
			      d->drawing_area->window);
  d->wall = ant_gfx_get_color(gdkcolormap, 0x6000, 0x6000, 0x6000, 
			      d->drawing_area->window);
  d->ant = ant_gfx_get_color(gdkcolormap, 0xafff, 0x1ddd, 0x0a00, 
			     d->drawing_area->window);
  d->wood = ant_gfx_get_color(gdkcolormap, 0xffff, 0x8000, 0, 
			      d->drawing_area->window);
  d->wood_fix = ant_gfx_get_color(gdkcolormap, 0xf000, 0x6000, 0, 
				  d->drawing_area->window);
  d->font = gdk_font_load(d->world->rules->font_name);
}

void gdk_draw_cross(GdkDrawable *d, GdkGC *gc,
		    gint x, gint y, gint sx, gint sy)

{
  gdk_draw_rectangle(d, gc, FALSE, x, y, sx, sy);
  gdk_draw_line(d, gc, x, y, x + sx, y + sy);
  gdk_draw_line(d, gc, x + sx, y, x, y + sy);
}

void ant_gfx_draw_background(GdkDrawable *d, GSList *back, gint step,
			     gint x1, gint y1, gint x2, gint y2)
{
  int i, nb, ty, tx, sx, sy;
  GSList *l;

  nb = g_slist_length(back);
  if (nb > 1)
    {
      for (l = back, i = 0; l != NULL; l = g_slist_next(l), i++)
	for (ty = y1; ty < y2; ty += step)
	  for (tx = x1; tx < x2; tx += step)
	    if (((tx + ty) / step) % nb == i)
	      {
		sx = MIN(step, x2 - tx);
		sy = MIN(step, y2 - ty);
		gdk_draw_rectangle(d, l->data, TRUE, 
				   tx, ty, sx, sy);
	      }
    }
  else
    gdk_draw_rectangle(d, back->data, TRUE, x1, y1, x2 - x1, y2 - y1);
}


void ant_gfx_draw_cell(ant_gfx_drawing_t *d, int x, int y)
{
  content_t *p;
  GSList *back = NULL;
  int nb_ants = 0;
  int has_wood = 0;
  char buf[5];
  int x1, y1, x2, y2;

  if (d == NULL)
    return;

  x1 = x * d->cs + 1;
  y1 = y * d->cs + 1;
  x2 = (x + 1) * d->cs - 1;
  y2 = (y + 1) * d->cs - 1;

  for (p = MAP(*d->world, x, y); p != NULL; p = p->next)
    {
      switch (p->type)
	{
	case cont_home:
	  back = g_slist_prepend(back, d->home);
	  break;
	case cont_entry:
	  back = g_slist_prepend(back, d->entry);
	  break;
	case cont_wall:
	  back = g_slist_prepend(back, d->wall);
	  break;
	case cont_ant:
	  nb_ants++;
	  break;
	case cont_exit:
	  back = g_slist_prepend(back, d->exit);
	  break;
	case cont_wood:
	  if (p->param >= 0)
	    has_wood = 1;
	  else
	    has_wood = 2;
	  break;
	}
    }

  if (back == NULL)
    back = g_slist_prepend(back, d->back);
  ant_gfx_draw_background(d->pixmap, back, 
			  d->world->rules->paving,
			  x1, y1, x2 + 1, y2 + 1);
  g_slist_free(back);

  if (has_wood)
    gdk_draw_rectangle(d->pixmap, d->wood, TRUE, 
		       x1 + d->cs / 4, y1 + d->cs / 4, 
		       (d->cs - 1) / 2, (d->cs - 1) / 2);
  if (has_wood == 2)
    {
      gdk_draw_rectangle(d->pixmap, d->wood_fix, TRUE, 
			 x1 + d->cs / 4, y1 + d->cs / 4, 
			 (d->cs - 1) / 2, (d->cs - 1) / 2);
      gdk_draw_cross(d->pixmap, d->line, x1 + d->cs / 4, y1 + d->cs / 4, 
		     (d->cs - 1) / 2, (d->cs - 1) / 2);
    }
  if (nb_ants)
    {
      g_snprintf(buf, 5, "%d", nb_ants);
      gdk_draw_string(d->pixmap, d->font, d->ant, 
		      x1 + 1, y2 - 1,
		      buf);
    }
}

void ant_gfx_refresh_world(ant_gfx_drawing_t *d)
{
  int x, y;

  if (d == NULL)
    return;

  /* Clear ground */
  gdk_draw_rectangle(d->pixmap, d->back, TRUE, 
		     0, 0, 
		     d->world->size.x * d->cs, d->world->size.y * d->cs);

  /* Draw main gird */
  for (x = 0; x <= d->world->size.x; x++)
    gdk_draw_line(d->pixmap, d->line,  
		  x * d->cs, 0,
		  x * d->cs, d->world->size.y * d->cs);
  for (y = 0; y <= d->world->size.y; y++)
    gdk_draw_line(d->pixmap, d->line,
		  0, y * d->cs,
		  d->world->size.x * d->cs, y * d->cs);
  
  /* Put objects */
  for (x = 0; x < d->world->size.x; x++)
    for (y = 0; y < d->world->size.y; y++)
      ant_gfx_draw_cell(d, x, y);

  gdk_draw_pixmap(d->drawing_area->window,
                  d->drawing_area->style->fg_gc[GTK_WIDGET_STATE(d->drawing_area)],
                  d->pixmap,
                  0, 0,
                  0, 0,
                  d->world->size.x * d->cs, d->world->size.y * d->cs);
}

gint ant_configure_event(GtkWidget *w, GdkEventConfigure *e, 
			 ant_gfx_drawing_t *d)
{
  if (d->pixmap == NULL)
    {
      /* Create the main board */
      d->pixmap = gdk_pixmap_new(d->drawing_area->window, 
				 d->world->size.x * d->cs + 1, 
				 d->world->size.y * d->cs + 1, -1);
      /* Allocate colors */
      ant_gfx_get_colors(d);
    }
  ant_gfx_refresh_world(d);
  return TRUE;
}

/* Redraw the screen from the backing pixmap */
gint ant_expose_event(GtkWidget *widget, GdkEventExpose *e, 
		      ant_gfx_drawing_t *d)
{
  gdk_draw_pixmap(widget->window,
                  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                  d->pixmap,
                  e->area.x, e->area.y,
                  e->area.x, e->area.y,
                  e->area.width, e->area.height);
  return FALSE;
}

GtkWidget *ant_make_mainbox(GtkWidget *window, world_t *w)
{
  GtkWidget *table, *frame, *vbox, *hbox;
  ant_gfx_drawing_t *d;

  ALLOC(d);
  d->world = w;
  d->cs = d->world->rules->cell_size;
  w->gfx_data = d;

  /* Create boxes */
  hbox = gtk_hbox_new(FALSE, 5);
  vbox = gtk_vbox_new(FALSE, 5);

  /* Stats */
  table = gtk_table_new(5, 4, FALSE);
  d->nbl = ant_gfx_create_entry(table, "Ants born: ", &(w->nb_born), 
				NULL, 0, 0);
  d->nbc = ant_gfx_create_entry(table, "Number of cycles: ", &(w->nb_live), 
				NULL, 2, 0);
  d->nbd = ant_gfx_create_entry(table, "Ants exited: ", &(w->nb_die), 
				&(w->nb_die), 0, 1);
  d->nba = ant_gfx_create_entry(table, "Ants living yet: ", &(w->nb_ant), 
				&(w->nb_die), 2, 1);  
  d->exit_ratio = ant_gfx_create_progress(table, "Exit ratio: ", 
					  &(w->nb_die), &(w->nb_born), 
					  &(w->nb_die), 2);
  d->wood_home = ant_gfx_create_entry(table, "Wood at home: ", &(w->wood_home),
				      &(w->wood_total), 0, 3);
  d->wood_map = ant_gfx_create_entry(table, "Wood on map: ", &(w->wood_map),
				     &(w->wood_total), 2, 3);
  d->wood_ratio = ant_gfx_create_progress(table, "Wood ratio: ", 
					  &(w->wood_home), &(w->wood_total), 
					  &(w->wood_total), 4);
  gtk_widget_show(table);
  frame = gtk_frame_new("Statistics");
  gtk_container_add(GTK_CONTAINER(frame), table);
  gtk_widget_show(frame);
  gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);

  /* Create main box */
  d->drawing_area = gtk_drawing_area_new();
  d->pixmap = NULL;
  gtk_drawing_area_size(GTK_DRAWING_AREA(d->drawing_area),
			w->size.x * w->rules->cell_size + 1, 
			w->size.y * w->rules->cell_size + 1);
  gtk_signal_connect(GTK_OBJECT(d->drawing_area), "expose_event",
		     (GtkSignalFunc) ant_expose_event, d);
  gtk_signal_connect(GTK_OBJECT(d->drawing_area), "configure_event",
		     (GtkSignalFunc) ant_configure_event, d);
  gtk_widget_show(d->drawing_area);
  frame = gtk_frame_new("The map");
  gtk_container_add(GTK_CONTAINER(frame), d->drawing_area);
  gtk_widget_show(frame);
  gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);

  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
  gtk_widget_show(vbox);
  gtk_widget_show(hbox);
  return hbox;
}

/* Update a map cell */
void ant_gtk_redraw_cell(world_t *w, int x, int y)
{
  ant_gfx_drawing_t *d;

  if (!ant_gtk_ready)
    return;
  gdk_threads_enter();
  d = w->gfx_data;
  ant_gfx_draw_cell(w->gfx_data, x, y);
  gdk_draw_pixmap(d->drawing_area->window,
                  d->drawing_area->style->fg_gc[GTK_WIDGET_STATE(d->drawing_area)],
                  d->pixmap,
                  x * d->cs, y * d->cs,
                  x * d->cs, y * d->cs,
                  d->cs, d->cs);
  gdk_threads_leave();
  ant_gtk_stats_refresh(w);
}

/* Redraw the full world */
void ant_gtk_redraw_world(world_t *w)
{
  ant_gfx_drawing_t *d;

  if (!ant_gtk_ready)
    return;
  gdk_threads_enter();
  d = w->gfx_data;
  ant_gfx_refresh_world(d);
  gdk_threads_leave();
  ant_gtk_stats_refresh(w);
}
