/*
** The thread part of GTK+ interface.
**
** $Id: ant_gtk_mutex.c,v 1.6 2001/02/21 21:07:25 kilobug Exp $
*/

#include "ant_gtk.h"

GMutex *ant_the_gtk_mutex = NULL;
GCond *ant_gtk_action_cond = NULL;

/* For the main program: wait if the GTK engine ask for pause. */
void ant_gtk_action_begin(world_t *w)
{
  g_mutex_lock(ant_the_gtk_mutex);
  g_mutex_unlock(ant_the_gtk_mutex);  
}

void ant_gtk_action_end(world_t *w)
{
  static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
  
  g_static_mutex_lock(&mutex);
  g_mutex_lock(ant_the_gtk_mutex);
  g_mutex_unlock(ant_the_gtk_mutex);  
  g_cond_signal(ant_gtk_action_cond);
  g_static_mutex_unlock(&mutex);
}

/* Allocate the mutex */
void ant_gtk_thread_init_mutex()
{
  ant_the_gtk_mutex = g_mutex_new();
  ant_gtk_action_cond = g_cond_new();
}

/* Enter pause mode */
void ant_gtk_thread_lock()
{
  g_mutex_trylock(ant_the_gtk_mutex);
}

void ant_gtk_lock_callback(GtkWidget *w, gpointer data)
{
  ant_set_status("Entering pause mode.");
  ant_gtk_thread_lock();
}

/* Leave pause mode */
void ant_gtk_thread_unlock()
{
  g_mutex_unlock(ant_the_gtk_mutex);
}

void ant_gtk_unlock_callback(GtkWidget *w, gpointer data)
{
  ant_set_status("Leaving pause mode.");
  ant_gtk_thread_unlock();
}

/* Run just a single step */
void ant_gtk_step_callback(GtkWidget *w, gpointer data)
{
  g_mutex_trylock(ant_the_gtk_mutex);
  g_cond_wait(ant_gtk_action_cond, ant_the_gtk_mutex);
}
