/*
** The GTK+ interface: interface creating functions.
**
** $Id: ant_gtk_stats.c,v 1.1 2001/02/22 02:06:57 kilobug Exp $
*/

#include "ant_gtk.h"
#include "ant_gtk_mainbox.h"

/* Create an entry with a label and link it to the table */
ant_gfx_entry_t *ant_gfx_create_entry(GtkWidget *table, const char *label,
				      int *val, int *dep, int x, int y)
{
  ant_gfx_entry_t *res;

  ALLOC(res);
  res->curval = 0;
  res->val = val;
  res->dep = dep;

  res->lbl = gtk_label_new(label);
  gtk_label_set_justify(GTK_LABEL(res->lbl), GTK_JUSTIFY_LEFT);
  gtk_table_attach(GTK_TABLE(table), res->lbl, 
		   x, x + 1, y, y + 1,
		   GTK_FILL | GTK_EXPAND , GTK_FILL | GTK_EXPAND, 2, 2);

  res->entry = GTK_ENTRY(gtk_entry_new_with_max_length(6));
  gtk_entry_set_editable(res->entry, FALSE);
  gtk_entry_set_text(res->entry, "0");
  gtk_widget_set_usize(GTK_WIDGET(res->entry), 80, -2);
  gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(res->entry),
		   x + 1, x + 2, y, y + 1,
		   GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
  ant_gfx_entry_update(res);
  return res;
}

/* Create an progress bar with a label and link it to the table */
ant_gfx_progress_t *ant_gfx_create_progress(GtkWidget *table, const char *lbl,
					    int *val, int *max,
					    int *dep, int y)
{
  ant_gfx_progress_t *res;

  ALLOC(res);

  res->curval = res->curmax = -42;
  res->val = val;
  res->max = max;
  res->dep = dep;

  res->lbl = gtk_label_new(lbl);
  gtk_table_attach_defaults(GTK_TABLE(table), res->lbl, 0, 1, y, y + 1);

  res->progress = GTK_PROGRESS(gtk_progress_bar_new());
  gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(res->progress), 
			    1, 4, y, y + 1);
  gtk_progress_set_format_string(res->progress, "%p %%");
  gtk_progress_set_show_text(res->progress, TRUE);

  ant_gfx_progress_update(res);
  return res;
}

/* Update an entry */
void ant_gfx_entry_update(ant_gfx_entry_t *entry)
{
  static char buf[10];

  if ((entry->dep == NULL) || (*(entry->dep)))
    {
      gtk_widget_show(entry->lbl);
      gtk_widget_show(GTK_WIDGET(entry->entry));
    }
  else
    {
      gtk_widget_hide(entry->lbl);
      gtk_widget_hide(GTK_WIDGET(entry->entry));
      return;
    }

  if (*(entry->val) == entry->curval)
    return;
  entry->curval = *(entry->val);
  g_snprintf(buf, 10, "%d", entry->curval);
  gtk_entry_set_text(entry->entry, buf);
}

/* Update a progress bar */
void ant_gfx_progress_update(ant_gfx_progress_t *p)
{
  if ((p->dep == NULL) || (*(p->dep)))
    {
      gtk_widget_show(p->lbl);
      gtk_widget_show(GTK_WIDGET(p->progress));
    }
  else
    {
      gtk_widget_hide(p->lbl);
      gtk_widget_hide(GTK_WIDGET(p->progress));
      return;
    }

  if ((*(p->val) == p->curval) && (*(p->max) == p->curmax))
    return;
  p->curval = *(p->val);
  p->curmax = *(p->max);
  gtk_progress_configure(p->progress, MAX(p->curval, 0), 0, MAX(p->curmax, 1));
}

/* Update the statistics */
void ant_gtk_stats_refresh(world_t *w)
{
  ant_gfx_drawing_t *d;

  gdk_threads_enter();
  d = w->gfx_data;
  /* Main stats */
  ant_gfx_entry_update(d->nbl);
  ant_gfx_entry_update(d->nbc);

  /* Exit infos */
  ant_gfx_entry_update(d->nba);
  ant_gfx_entry_update(d->nbd);
  ant_gfx_progress_update(d->exit_ratio);

  /* Wood infos */
  ant_gfx_entry_update(d->wood_home);
  ant_gfx_entry_update(d->wood_map);
  ant_gfx_progress_update(d->wood_ratio);

  gdk_threads_leave();
}

