/*
** Contains the main function.
**
** $Id: ant_main.c,v 1.13 2001/02/21 20:42:46 kilobug Exp $
*/

#include "ant.h"
#include "ant_private.h"

/* The loop creating ants */
void *ant_create_ants(world_t *w)
{
  while (TRUE)
    ant_create_ant(w);
}

/* The event loop in non-GTK mode */
void ant_main_loop_default(world_t *w)
{
  while (TRUE)
    pause();
}

/* Main function for static linked clients */
error_t ant_main(int argc, char **argv, ant_callback_t *live, 
		 ant_callback_t *born, ant_callback_t *exit)
{
  world_t *world;
  pthread_t creator;

  world = ant_engine_init(argc, argv, live, born, exit);
  if ((world == NULL) || (live == NULL))
   return ERROR_PARAM;
  pthread_create(&creator, NULL, (void *) ant_create_ants, world);
  ant_main_loop(world);
  return ERROR_NONE;
}

/* Load a plugin and link its callbacks to the world */
void ant_load_plugin(const char *name, world_t *world)
{
  GModule *module;

  if (name == NULL)
    {
      fprintf(stderr, "No plugin specified. Aborting.\n");
      exit(1);
    }
  module = g_module_open(name, 0);
  if (module == NULL)
    {
      fprintf(stderr, "g_module_open: %s.\n", g_module_error());
      exit(1);
    }
  g_module_make_resident(module);
  if (!g_module_symbol(module, "ant_live", (gpointer *) &(world->live)))
    {
      fprintf(stderr, "%s: no such symbol: ant_live.\n", name);
      exit(1);
    }
  if (!g_module_symbol(module, "ant_born", (gpointer *) &(world->born)))
    {
      fprintf(stderr, "%s: no such symbol: ant_born.\n", name);
      exit(1);
    }
  if (!g_module_symbol(module, "ant_exit", (gpointer *) &(world->exit)))
    {
      fprintf(stderr, "%s: no such symbol: ant_exit.\n", name);
      exit(1);
    }
}

/* Main function for the "ants" .so dynamic-loader client */
error_t ant_main_module(int argc, char **argv)
{
  world_t *world;
  pthread_t creator;

  ant_run_level = lvl_plugin;
  world = ant_engine_init(argc, argv, NULL, NULL, NULL);
  ant_load_plugin(world->rules->plugin_name, world);
  pthread_create(&creator, NULL, (void *) ant_create_ants, world);
  ant_main_loop(world);
  return ERROR_NONE;
}


