/*
** Command-line parser.
**
** $Id: cmdline.c,v 1.2 2001/02/21 20:42:46 kilobug Exp $
*/

#include "ant_private.h"

typedef struct ant_cmd_line_act_s
{
  const char *s;  
  enum { clact_discard, clact_help, clact_conf, clact_noconf, clact_unknown,
	 clact_version, clact_plugin } act;
  const char *help;
  ant_run_level_t level;
} ant_cmd_line_act_t;

ant_cmd_line_act_t ant_cmdline[] = 
   { { "-h", clact_help, "Display this help", lvl_lib },
     { "--help", clact_help, "Display this help", lvl_lib },
     { "-v", clact_version, "Display the version number", lvl_lib },
     { "--no-conf", clact_noconf, "Load no conf file: use defaults", lvl_lib },
     { "-f", clact_conf, "Load specified conf file (default ./antrc)", 
       lvl_lib },
     { "-p", clact_plugin, "Name of the ant code plugin (.so)", lvl_plugin },
     { NULL, clact_unknown, NULL, lvl_lib } };

/* Display an help message */
void ant_print_help(const char *self)
{
  int i, l;

  printf("Syntax: %s [options]\n\n", self);
  for (i = 0; ant_cmdline[i].s != NULL; i++)
    if (ant_cmdline[i].level <= ant_run_level)
      {
	l = strlen(ant_cmdline[i].s);
	printf("  %s", ant_cmdline[i].s);
	for (; l < 20; l++)
	  putchar(' ');
	printf(": %s\n", ant_cmdline[i].help);
      }
  putchar('\n');
}

/* The main command line parser */
void ant_parse_cmdline(int argc, const char **argv, rules_t *res)
{
  int i, j;

  for (i = 1; i < argc; i++)
    {
      for (j = 0; ant_cmdline[j].s != NULL; j++)
	if ((!strcmp(ant_cmdline[j].s, argv[i])) && 
	    (ant_cmdline[j].level <= ant_run_level))
	  break;
      switch (ant_cmdline[j].act)
	{
	case clact_help:
	  ant_print_help(argv[0]);
	  exit (0);
	case clact_version:
	  printf("%s version %s\n", PACKAGE, VERSION);
	  break;
	case clact_noconf:
	  SAFE_FREE(res->conf_name);
	  break;
	case clact_conf:
	  if (++i == argc)
	    printf("Warning: %s requires a parameter.\n", argv[i - 1]);
	  else
	    {
	      SAFE_FREE(res->conf_name);
	      res->conf_name = strdup(argv[i]);
	    }
	  break;
	case clact_plugin:
	  if (++i == argc)
	    printf("Warning: %s requires a parameter.\n", argv[i - 1]);
	  else
	    {
	      SAFE_FREE(res->plugin_name);
	      res->plugin_name = strdup(argv[i]);
	    }
	  break;
	case clact_discard:
	  break;
	case clact_unknown:
	  printf("Warning: unkwon parameter %s. Try --help.\n", argv[i]);
	  break;
	}
    }
}

