/*
** Contains the initialisation function.
**
** $Id: init.c,v 1.23 2001/02/24 20:50:54 kilobug Exp $
*/

#include "ant.h"
#include "ant_private.h"

ant_run_level_t ant_run_level = lvl_lib;

/* Load rules from file or default */
rules_t *ant_load_rules(int argc, const char **argv)
{
  rules_t *res;
  char *s, *s2;

  ALLOC(res);
  /* Set defaults */
  res->font_name = strdup("-adobe-courier-bold-r-normal-*-12-*-*-*-*-*-*-*");
  res->map_name = strdup("default.map");
  res->conf_name = strdup("antrc");
  res->plugin_name = NULL;
  res->cell_size = 16;
  res->paving = 3;
  res->size.y = res->size.x = 10;
  res->nb_ant_total = -1;
  res->nb_ant_max = 242;
  res->max_by_cell = 1;
  res->jump_size = 1;
  res->create.min = 1000;
  res->create.max = 3000;
  res->live.min = 500;
  res->live.max = 1000;
  res->entry.y = res->entry.x = 0;
  res->data = NULL;
  ant_parse_cmdline(argc, argv, res);
  if (res->conf_name == NULL)
    return res;
  ant_parse_conf_file(res, argv[0]);
  if (!g_path_is_absolute(res->map_name))
    {
      s = g_dirname(res->conf_name);
      s2 = g_strjoin(G_DIR_SEPARATOR_S, s, res->map_name, NULL);
      SAFE_FREE(s);
      SAFE_FREE(res->map_name);
      res->map_name = s2;
    }
  return res;
}

/* Ant engine initializer: parse command line, load rules, create the world */
world_t *ant_engine_init(int argc, char **argv, ant_callback_t *live, 
			 ant_callback_t *born, ant_callback_t *die)
{
  rules_t *rules;
  world_t *world;

#ifndef G_THREADS_ENABLED
#error "No threads support in GLIB, sorry."
#endif
#ifdef G_THREADS_IMPL_NONE
#error "No threads support in GLIB, sorry."
#endif
  g_thread_init(NULL);
  if (!g_thread_supported())
    {
      fprintf(stderr, "Can't init g_thread, aborting.\n");
      exit (2);
    }

  srandom(time(NULL));

  ants_mutex = g_mutex_new();
  ant_main_loop_init(&argc, &argv);
  rules = ant_load_rules(argc, (const char **) argv);
  if (rules == NULL)
    exit (1);
  world = ant_create_world(rules);
  ant_load_map(rules->map_name, world);
  if (world->entry == NULL)
    {
      fprintf(stderr, "Warning: no entry on map - using rule default.\n");
      ant_do_add_item(world, &(rules->entry), cont_entry, 0);
    }
  world->live = live;
  world->born = born;
  world->exit = die;
  return world;
}

