/*
** Load maps.
**
** $Id: map.c,v 1.5 2001/02/24 15:09:32 kilobug Exp $
*/

#include "ant.h"
#include "ant_private.h"
#include "ant_parser.h"

#define MAP_SCOPE_NONE 0
#define MAP_SCOPE_TYPE 1
#define MAP_SCOPE_WOOD 2

#ifndef G_TOKEN_SEMICOLON
#define G_TOKEN_SEMICOLON ';'
#endif

/* Get a paramter as an int or a symbol */
int ant_map_get_param(GScanner *scan, int scope)
{
  GTokenType type;

  g_scanner_set_scope(scan, scope);
  type = g_scanner_get_next_token(scan);
  if (type == G_TOKEN_INT)
    return g_scanner_cur_value(scan).v_int;
  if (type == G_TOKEN_SYMBOL)
    return *((int *) g_scanner_cur_value(scan).v_symbol);
  g_scanner_unexp_token(scan, G_TOKEN_INT, NULL, NULL, NULL, NULL,
			TRUE);
  return 0;
}

/* Load a value: (x, y) keyword [param]; */
error_t ant_map_load_value(GScanner *scan, world_t *world)
{
  coord_t where;
  content_type_t cont;
  int param = 0;

  PARSER_CHECK_TYPE(g_scanner_get_next_token(scan), G_TOKEN_INT);
  where.x = g_scanner_cur_value(scan).v_int;
  PARSER_CHECK_TYPE(g_scanner_get_next_token(scan), G_TOKEN_COMMA);
  PARSER_CHECK_TYPE(g_scanner_get_next_token(scan), G_TOKEN_INT);
  where.y = g_scanner_cur_value(scan).v_int;
  PARSER_CHECK_TYPE(g_scanner_get_next_token(scan), G_TOKEN_RIGHT_PAREN);
  
  g_scanner_set_scope(scan, MAP_SCOPE_TYPE);
  PARSER_CHECK_TYPE(g_scanner_get_next_token(scan), G_TOKEN_SYMBOL);
  cont = *((int *) g_scanner_cur_value(scan).v_symbol);
  if (cont == cont_wood)
    param = ant_map_get_param(scan, MAP_SCOPE_WOOD);
  g_scanner_set_scope(scan, MAP_SCOPE_NONE);

  PARSER_CHECK_TYPE(g_scanner_get_next_token(scan), G_TOKEN_SEMICOLON);

  if ((where.x >= world->size.x) || (where.x < 0)
      || (where.y >= world->size.y) || (where.y < 0))      
    {
      g_scanner_error(scan, "coordinates our of range, statement discarded");
      return ERROR_PARSE;
    }
  ant_do_add_item(world, &where, cont, param);
  return ERROR_NONE;
}

/* Create the scopes */
void ant_map_init_scopes(GScanner *scan)
{
  static content_type_t wall = cont_wall;
  static content_type_t exit = cont_exit;
  static content_type_t wood = cont_wood;
  static content_type_t entry = cont_entry;
  static content_type_t home = cont_home;
  static int wood_fix = -42;

  g_scanner_scope_add_symbol(scan, MAP_SCOPE_TYPE, "wall", &wall);  
  g_scanner_scope_add_symbol(scan, MAP_SCOPE_TYPE, "exit", &exit);  
  g_scanner_scope_add_symbol(scan, MAP_SCOPE_TYPE, "wood", &wood);  
  g_scanner_scope_add_symbol(scan, MAP_SCOPE_TYPE, "entry", &entry);  
  g_scanner_scope_add_symbol(scan, MAP_SCOPE_TYPE, "home", &home);  
  g_scanner_scope_add_symbol(scan, MAP_SCOPE_WOOD, "fix", &wood_fix);
}

/* Parse a map file */
void ant_load_map(const char *name, world_t *world)
{
  GScanner *scan;
  int fd;
  GTokenType type;

  fd = open(name, O_RDONLY);
  if (fd == -1)
    {
      fprintf(stderr, "Warning: can't open map file (%s): %s\n", name, 
	      strerror(errno));
      return;
    }
  scan = g_scanner_new(NULL);
  scan->config->int_2_float = FALSE;
  scan->input_name = name;

  ant_map_init_scopes(scan);
  g_scanner_set_scope(scan, MAP_SCOPE_NONE);

  g_scanner_input_file(scan, fd);
  while ((type = g_scanner_get_next_token(scan)) != G_TOKEN_EOF)
    {
      if (type == G_TOKEN_LEFT_PAREN)
	ant_map_load_value(scan, world);
      else
	g_scanner_unexp_token(scan, G_TOKEN_LEFT_PAREN, NULL, NULL, NULL, NULL,
			      TRUE);	
    }
  g_scanner_destroy(scan);
}
