/*
** The main parser.
**
** $Id: parser.c,v 1.14 2001/02/24 20:54:06 kilobug Exp $
*/

#include "ant.h"
#include "ant_private.h"
#include "ant_parser.h"

#define RULE_OFFSET(ID) G_STRUCT_OFFSET(rules_t, ID)

struct ant_parser_s ant_config_file[] = 
{ { "general", "mapsize", RULE_OFFSET(size), pt_coord, SCOPE_SIZE },
  { "general", "max_ants", RULE_OFFSET(nb_ant_max), pt_int, SCOPE_NUM },
  { "general", "total_ants", RULE_OFFSET(nb_ant_total), pt_int, SCOPE_NUM },
  { "general", "max_by_cell", RULE_OFFSET(max_by_cell), pt_int, SCOPE_NUM },
  { "general", "entry_point", RULE_OFFSET(entry), pt_coord, SCOPE_POS },
  { "general", "map", RULE_OFFSET(map_name), pt_string, SCOPE_NONE },
  { "general", "jump_size", RULE_OFFSET(jump_size), pt_int, SCOPE_NUM },
  { "gtk", "cell_size", RULE_OFFSET(cell_size), pt_int, SCOPE_NONE },
  { "gtk", "paving", RULE_OFFSET(paving), pt_int, SCOPE_NONE },
  { "gtk", "font", RULE_OFFSET(font_name), pt_string, SCOPE_NONE },
  { "creating", "max_ants", RULE_OFFSET(nb_ant_max), pt_int, SCOPE_NUM },
  { "creating", "total_ants", RULE_OFFSET(nb_ant_total), pt_int, SCOPE_NUM },
  { "creating", "delay", RULE_OFFSET(create), pt_delay, SCOPE_DELAY },
  { "creating", "position", RULE_OFFSET(entry), pt_coord, SCOPE_POS },
  { "running", "delay", RULE_OFFSET(live), pt_delay, SCOPE_DELAY },
  { NULL, NULL, 0, pt_none, SCOPE_NONE } };

/* A [ was find: change current section name */
void ant_parser_get_section(GScanner *scan, char **section)
{
  SAFE_FREE(*section);
  if (g_scanner_get_next_token(scan) != G_TOKEN_IDENTIFIER)
    {
      g_scanner_unexp_token(scan, G_TOKEN_IDENTIFIER, NULL, NULL, NULL, NULL,
			    TRUE);
      return;
    }
  *section = strdup(g_scanner_cur_value(scan).v_string);
  if (g_scanner_get_next_token(scan) != G_TOKEN_RIGHT_BRACE)
    {
      g_scanner_unexp_token(scan, G_TOKEN_RIGHT_BRACE, NULL, NULL, NULL, NULL,
			    TRUE);
      SAFE_FREE(section);
    }
}

/* Add contsants */
void ant_parse_init_scopes(GScanner *scan)
{
  static int inf = -1;
  static int zero = 0;
  static coord_t topleft = { 0, 0 };
  static coord_t topright = { -1, 0 };
  static coord_t bottomleft = { 0, -1 };
  static coord_t bottomright = { -1, -1 };
  static coord_t center = { -2, -2 };
  static int middle = -2;
  static int corner = -1;
  static delay_t instant = { 0, 0 };

  g_scanner_scope_add_symbol(scan, SCOPE_NUM, "inf", &inf);
  g_scanner_scope_add_symbol(scan, SCOPE_NUM, "infinity", &inf);
  g_scanner_scope_add_symbol(scan, SCOPE_NUM, "infinite", &inf);
  g_scanner_scope_add_symbol(scan, SCOPE_NUM, "zero", &zero);
  g_scanner_scope_add_symbol(scan, SCOPE_NUM, "none", &zero);
  g_scanner_scope_add_symbol(scan, SCOPE_NUM, "void", &zero);
  g_scanner_scope_add_symbol(scan, SCOPE_SIZE, "zero", &topleft);
  g_scanner_scope_add_symbol(scan, SCOPE_SIZE, "none", &topleft);
  g_scanner_scope_add_symbol(scan, SCOPE_SIZE, "void", &topleft);
  g_scanner_scope_add_symbol(scan, SCOPE_POS, "zero", &topleft);
  g_scanner_scope_add_symbol(scan, SCOPE_POS, "topleft", &topleft);
  g_scanner_scope_add_symbol(scan, SCOPE_POS, "topright", &topright);
  g_scanner_scope_add_symbol(scan, SCOPE_POS, "bottomright", &bottomright);
  g_scanner_scope_add_symbol(scan, SCOPE_POS, "bottomleft", &bottomleft);
  g_scanner_scope_add_symbol(scan, SCOPE_POS, "center", &center);
  g_scanner_scope_add_symbol(scan, SCOPE_POS_ELEM, "middle", &middle);
  g_scanner_scope_add_symbol(scan, SCOPE_POS_ELEM, "left", &zero);
  g_scanner_scope_add_symbol(scan, SCOPE_POS_ELEM, "top", &zero);
  g_scanner_scope_add_symbol(scan, SCOPE_POS_ELEM, "right", &corner);
  g_scanner_scope_add_symbol(scan, SCOPE_POS_ELEM, "bottom", &corner);
  g_scanner_scope_add_symbol(scan, SCOPE_DELAY, "none", &instant);
}

/* Check the validity of the result, and reslove special values */
void ant_ensure_above(int *what, int min, const char *name, int val)
{
  if (*what < min)
    {
      fprintf(stderr, 
	      "=> Warning: %s is %d and should be above %d. Set to %d.\n", 
	      name, *what, min, val);
      *what = val;
    }
}

void ant_ensure_below(int *what, int max, const char *name, int val)
{
  if (*what > max)
    {
      fprintf(stderr, 
	      "=> Warning: %s is %d and should be below %d. Set to %d.\n", 
	      name, *what, max, val);
      *what = val;
    }
}

void ant_ensure_range(int *what, int min, int max, const char *name)
{
  ant_ensure_above(what, min, name, min);
  ant_ensure_below(what, max, name, max);
}

void ant_check_validity(rules_t *rules)
{
#ifdef __DEBUG__
  fprintf(stderr, "Performing sanity checks...\n");
#endif
  ant_ensure_above(&(rules->size.x), 1, "width of the board", 10);
  ant_ensure_above(&(rules->size.y), 1, "height of the board", 10);
  if (rules->entry.x == -1)
    rules->entry.x = rules->size.x - 1;
  if (rules->entry.y == -1)
    rules->entry.y = rules->size.y - 1;
  if (rules->entry.x == -2)
    rules->entry.x = rules->size.x / 2 - 1;
  if (rules->entry.y == -2)
    rules->entry.y = rules->size.y / 2 - 1;
  ant_ensure_range(&(rules->entry.x), 0, rules->size.x - 1, 
		   "starting X position");
  ant_ensure_range(&(rules->entry.y), 0, rules->size.y - 1, 
		   "starting Y position");
  ant_ensure_above(&(rules->cell_size), 5, "graphical cell size", 10);
  ant_ensure_above(&(rules->paving), 1, "graphical subcell size", 2);
  ant_ensure_above(&(rules->create.min), 0, "creation minimum delay", 1);
  ant_ensure_above(&(rules->create.max), 0, "creation maximum delay", 10);
  ant_ensure_above(&(rules->create.max), rules->create.min, 
		   "creation maximum delay", rules->create.min);
  ant_ensure_above(&(rules->live.min), 1, "cycle minimum delay", 1);
  ant_ensure_above(&(rules->live.max), 1, "cycle maximum delay", 10);
  ant_ensure_above(&(rules->live.max), rules->live.min, 
		   "cycle maximum delay", rules->live.min);  
}

/* Get the true name (look in the share/Ants if not current directory) */
int ant_get_conf_file(rules_t *rules, const char *self)
{
  char *s = NULL, *base = NULL;
  int fd;

  fd = open(rules->conf_name, O_RDONLY);
  if (fd != -1)
    return fd;
  if (!g_path_is_absolute(rules->conf_name))
    {
      base = g_dirname(self);      
      s = g_strjoin(G_DIR_SEPARATOR_S, base, "..", "share", PACKAGE, 
		    rules->conf_name, NULL);
      SAFE_FREE(base);
#ifdef __DEBUG__
      fprintf(stderr, "Trying to open %s\n", s);
#endif
      fd = open(s, O_RDONLY);
      if (fd != -1)
	{
	  SAFE_FREE(rules->conf_name);
	  rules->conf_name = s;
	  return fd;
	}
      SAFE_FREE(s);
    }
  fprintf(stderr, "Warning: can't open config file (%s): %s\n",
	  rules->conf_name, strerror(errno));
  return -1;
}

/* Parse a config file */
void ant_parse_conf_file(rules_t *rules, const char *self)
{
  GScanner *scan;
  int fd;
  GTokenType type;
  char *section = NULL;
  char *s = NULL;

  if ((fd = ant_get_conf_file(rules, self)) == -1)
    return;
  scan = g_scanner_new(NULL);
  scan->config->int_2_float = FALSE;
  scan->input_name = s;
  g_scanner_input_file(scan, fd);
  ant_parse_init_scopes(scan);
  while ((type = g_scanner_get_next_token(scan)) != G_TOKEN_EOF)
    {
      if (type == G_TOKEN_LEFT_BRACE)
	{
	  ant_parser_get_section(scan, &section);
	  continue;
	}
      if (type == G_TOKEN_IDENTIFIER)
	{
	  ant_parser_get_value(scan, rules, section);
	  continue;
	}
      g_scanner_unexp_token(scan, G_TOKEN_IDENTIFIER, NULL, NULL, NULL, NULL,
			    TRUE);
    }
  g_scanner_destroy(scan);
  ant_check_validity(rules);
}
