/*
** $Id: laby.c,v 1.7 2001/02/24 22:55:01 kilobug Exp $
** More intelligent labyrinth solver.
*/

#include <ant.h>

typedef struct ant_data_s
{
  int dir;
  int tlock;
} ant_data_t;

error_t try_move(ant_t *ant, int dir)
{
  coord_t pos;
  error_t err;
  ant_data_t *d;

  pos.x = ant->pos.x;
  pos.y = ant->pos.y;
  switch (dir % 4)
    {
    case 0:
      pos.y++;
      break;
    case 1:
      pos.x--;
      break;
    case 2:
      pos.y--;
      break;
    case 3:
      pos.x++;
      break;      
    }
  err = ant_action(ant, ANT_ACT_MOVE, &pos);
  d = ant->data;
  if (err == ERROR_NONE)
    {
      d->tlock = 0;
      d->dir = dir % 4;
    }
  return err;
}

G_MODULE_EXPORT error_t ant_live(ant_t *ant)
{
  int front, left, right, rear, r;
  ant_data_t *d;

  d = ant->data;
  front = d->dir;
  left = d->dir + 1;
  rear = d->dir + 2;
  right = d->dir + 3;

  if (try_move(ant, front) == ERROR_NONE)
    return ERROR_NONE;
  r = random() % 2;
  if (try_move(ant, (r ? left : right)) == ERROR_NONE)
    return ERROR_NONE;
  if (try_move(ant, (r ? right : left)) == ERROR_NONE)
    return ERROR_NONE;
  if (((random() % MAX(8, 30 - d->tlock)) == 0) || 
      ((ant->pos.x == ant->world->rules->entry.x) &&
       (ant->pos.y == ant->world->rules->entry.y)))
    try_move(ant, rear);
  else
    d->tlock++;
  return ERROR_NONE;
}

G_MODULE_EXPORT error_t ant_born(ant_t *ant)
{
  ant_data_t *d;

  ALLOC(d);
  d->dir = random() % 4;
  d->tlock = 0;
  ant->data = d;
  ant_disp_msg("Hello, I'm a new ant, my name is %d and I'm at (%d, %d).\n", 
	       ant->id, ant->pos.x, ant->pos.y);
  return ERROR_NONE;
}

G_MODULE_EXPORT error_t ant_exit(ant_t *ant)
{
  SAFE_FREE(ant->data);
  ant_disp_msg("!!! Ant %d has found exit !!!\n", ant->id);
  return ERROR_NONE;
}

