/*
** $Id: prand.c,v 1.5 2001/02/24 22:55:01 kilobug Exp $
** Pseudo-random labyrinth solver.
*/

#include <ant.h>

typedef struct ant_data_s
{
  int dir;
} ant_data_t;


error_t try_move(ant_t *ant, int dir)
{
  coord_t pos;
  error_t err;

  pos.x = ant->pos.x;
  pos.y = ant->pos.y;
  switch (dir % 4)
    {
    case 0:
      pos.y++;
      break;
    case 1:
      pos.x--;
      break;
    case 2:
      pos.y--;
      break;
    case 3:
      pos.x++;
      break;      
    }
  err = ant_action(ant, ANT_ACT_MOVE, &pos);
  if (err == ERROR_NONE)
    ((ant_data_t *)ant->data)->dir = dir % 4;
  return err;
}

G_MODULE_EXPORT error_t ant_live(ant_t *ant)
{
  int front, left, right, rear;
  ant_data_t *d;

  d = ant->data;
  front = d->dir;
  left = d->dir + 1;
  rear = d->dir + 2;
  right = d->dir + 3;

  switch (random() % 9)
    {
    case 0: case 1: case 2: case 3:
      try_move(ant, front);
      break;
    case 4: case 5:
      try_move(ant, left);
      break;
    case 6: case 7:
      try_move(ant, right);
      break;
    case 8:
      try_move(ant, rear);
      break;
    }
  return ERROR_NONE;
}

G_MODULE_EXPORT error_t ant_born(ant_t *ant)
{
  ant_data_t *d;

  ALLOC(d);
  d->dir = random() % 4;
  ant->data = d;
  ant_disp_msg("Hello, I'm a new ant, my name is %d and I'm at (%d, %d).\n", 
	       ant->id, ant->pos.x, ant->pos.y);
  return ERROR_NONE;
}

G_MODULE_EXPORT error_t ant_exit(ant_t *ant)
{
  SAFE_FREE(ant->data);
  ant_disp_msg("!!! Ant %d has found exit !!!\n", ant->id);
  return ERROR_NONE;
}

