/*
** $Id: wood.c,v 1.5 2001/02/25 18:06:53 kilobug Exp $
** Wood picking client.
*/

#include <ant.h>

#define GROUP_SIZE	8

#define DIR_FRONT	0
#define DIR_LEFT	1
#define DIR_REAR	2
#define DIR_RIGHT	3

typedef struct proba_s
{
  int proba;
  int dir;
} proba_t;

proba_t proba[4] = { { 50, DIR_FRONT }, 
		     { 22, DIR_LEFT  }, 
		     { 22, DIR_RIGHT },
		     { 06, DIR_REAR  } };

typedef struct ant_data_s
{
  int dir;
  int gid;
  coord_t *home;
  ant_t *leader;
  int action, actid;
  coord_t *param;
} ant_data_t;

void compute_dir(ant_t *ant, coord_t *dest, int dir)
{
  ant_data_t *d;

  d = ant->data;
  dest->x = ant->pos.x;
  dest->y = ant->pos.y;
  switch ((d->dir + dir) % 4)
    {
    case 0:
      dest->y++;
      break;
    case 1:
      dest->x--;
      break;
    case 2:
      dest->y--;
      break;
    case 3:
      dest->x++;
      break;      
    }
  d->dir = (d->dir + dir) % 4;
}

void move_random(ant_t *ant)
{
  ant_data_t *d;
  int r, i;

  d = ant->data;
  r = random() % 100;
  for (i = 0; i < 4; i++)
    {
      r -= proba[i].proba;
      if (r < 0)
	break;
    }
  compute_dir(ant, d->param, proba[i].dir);
}

void find_wood(ant_t *ant)
{
  ant_data_t *d;

  d = ant->data;
  if (ant_find_item(ant->world, &(ant->pos), cont_wood) != NULL)
    {
      d->action = ANT_ACT_PUSH_MOVE;
      compute_dir(ant, d->param, DIR_REAR);
      return;
    }
  d->action = ANT_ACT_MOVE;
  move_random(ant);
}

void go_home(ant_t *ant)
{
  ant_data_t *d;
  int r;

  d = ant->data;
  d->action = ANT_ACT_PUSH_MOVE;
  if (ant_find_item(ant->world, &(ant->pos), cont_home) != NULL)
    {
      d->action = ANT_ACT_MOVE;
      move_random(ant);
      return;
    }
  if (d->home == NULL)
    {
      move_random(ant);
      return;
    }

  r = random() % 2;
  if (d->home->x == ant->pos.x)
    r = 0;
  if (d->home->y == ant->pos.y)
    r = 1;
  if (r == 0)
    {
      d->param->x = ant->pos.x;
      if (d->home->y > ant->pos.y)
	d->param->y = ant->pos.y + 1;
      else
	d->param->y = ant->pos.y - 1;
    }
  else
    {
      d->param->y = ant->pos.y;
      if (d->home->x > ant->pos.x)
	d->param->x = ant->pos.x + 1;
      else
	d->param->x = ant->pos.x - 1;
    }
}

int sync_group(ant_t *ant)
{
  GSList *l;
  ant_t *a;
  ant_data_t *d, *self;

  self = ant->data;
  for (l = ant->world->ants; l != NULL; l = g_slist_next(l))
    {
      if ((a = l->data) == NULL)
	continue;
      if ((d = a->data) == NULL)
	continue;
      if (self->gid == d->gid)
	if ((self->actid != d->actid) || (ant_dist(&(a->pos), &(ant->pos))))
	  return FALSE;
    }
  return TRUE;
}

int find_group(ant_t *ant)
{
  GSList *l, *orphans = NULL;
  ant_t *a;
  ant_data_t *d;
  int gid, i = 1;

  /* Get orphans list */
  ant_enter_critical();
  orphans = g_slist_prepend(orphans, ant);
  for (l = ant->world->ants; l != NULL; l = g_slist_next(l))
    if (l->data != ant)
      {
	if ((a = l->data) == NULL)
	  continue;
	if ((d = a->data) == NULL)
	  continue;
	if ((d->gid < 0) && (ant_dist(&(a->pos), &(ant->pos)) == 0))
	  {
	    orphans = g_slist_prepend(orphans, a);
	    if (i++ == GROUP_SIZE)
	      break;
	  }
      }
	
  /* Enough ants ? */
  if (i < GROUP_SIZE)
    {
      g_slist_free(orphans);
      ant_leave_critical();
      return FALSE;
    }

  /* Ok... */
  gid = ant_get_uniq_id();
  for (l = orphans; l != NULL; l = g_slist_next(l))
    {
      a = l->data;
      d = a->data;
      d->gid = gid;
      d->leader = ant;
    }
  d = ant->data;
  ALLOC(d->param);
  g_slist_free(orphans);
  ant_leave_critical();

  return TRUE;
}

G_MODULE_EXPORT error_t ant_live(ant_t *ant)
{
  ant_data_t *d, *dl;

  d = ant->data;
  /* No group yet ? */
  if (d->gid < 0)
    if (!find_group(ant))
      {
	ant_action(ant, ANT_ACT_NONE, NULL);
	return ERROR_NONE;
      }

  /* Not leader: just follow */
  if (d->leader != ant)
    {
      dl = d->leader->data;
      if (d->actid != dl->actid)
	{
	  if (ant_action(ant, dl->action, dl->param) != ERROR_FORBIDDEN)
	    d->actid = dl->actid;
	}
      else
	ant_action(ant, ANT_ACT_NONE, NULL);
      return ERROR_NONE;
    }

  /* Home here ? Remember */
  if ((d->home == NULL) && 
      (ant_find_item(ant->world, &(ant->pos), cont_home) != NULL))
    {
      ALLOC(d->home);
      d->home->x = ant->pos.x;
      d->home->y = ant->pos.y;
    }

  /* Wait group */
  if (!sync_group(ant))
    {
      ant_action(ant, ANT_ACT_NONE, NULL);
      return ERROR_NONE;
    }

  /* Choose action */
  ant_enter_critical();
  if (d->action == ANT_ACT_PUSH_MOVE)
    go_home(ant);
  else
    find_wood(ant);
  ant_leave_critical();

  /* Performs it */
  if (ant_action(ant, d->action, d->param) != ERROR_FORBIDDEN)
    d->actid++;
  return ERROR_NONE;
}

G_MODULE_EXPORT error_t ant_born(ant_t *ant)
{
  ant_data_t *d;

  ALLOC(d);
  d->dir = random() % 4;
  d->param = NULL;
  d->gid = -42;
  d->leader = NULL;
  d->home = NULL;
  d->action = ANT_ACT_NONE;
  d->actid = 0;
  ant->data = d;
  ant_disp_msg("Hello, I'm a new ant, my name is %d and I'm at (%d, %d).\n", 
	       ant->id, ant->pos.x, ant->pos.y);
  return ERROR_NONE;
}

G_MODULE_EXPORT error_t ant_exit(ant_t *ant)
{
  SAFE_FREE(ant->data);
  ant_disp_msg("!!! Ant %d has found exit !!!\n", ant->id);
  return ERROR_NONE;
}

